#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <poslib/poslib.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "main.h"
#include "thread.h"
#include "configfile.h"

#ifdef HAVE_NAMESER_H
  #include <nameser.h>
#endif
#ifdef HAVE_RESOLV_H
  #include <resolv.h>
#endif

bool txttype_flag = false, txttype_changed = false;

void
on_cmdQuery_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  if (query_busy) {
    query_stop();
    query_busy = false;
    gtk_label_set_text(GTK_LABEL(GTK_BIN(lookup_widget(frmMain, "cmdQuery"))->child), "Query!");
  } else {
    char *dom = gtk_entry_get_text(GTK_ENTRY(lookup_widget(frmMain, "txtDom"))),
         *server = gtk_entry_get_text(GTK_ENTRY(lookup_widget(frmMain, "txtServer"))),
         *type = gtk_entry_get_text(GTK_ENTRY(lookup_widget(frmMain, "txtType")));
    domainname qdom;
    uint16_t qtype;
    pthread_t tr;
    char typeb[16];
    char *ptr;
    bool do_rd = GTK_CHECK_MENU_ITEM(lookup_widget(mnuOptions, "mnuOptionsRecursion"))->active;

    /* clear result list */
    GtkList *gt = (GtkList *)lookup_widget(frmMain, "lstAnswer");
    gtk_list_clear_items (GTK_LIST(gt), 0, -1);

    try {
      qdom = dom;
      txt_to_addr(&qserver, server);
      /* add to recent file list */
      last_server = server;
      last_type = type;
      add_to_recentservers(server);
      add_to_recentqnames(dom);

      /* read qtype */
      ptr = strchr(type, ' ');
      if (!ptr) {
        if (strlen(type) >= sizeof(typeb)) throw PException("Query type string too long!");
        strcpy(typeb, type);
      } else {
        if (ptr - type >= (signed)sizeof(typeb)) throw PException("Query type string too long!");
        memcpy(typeb, type, ptr - type);
        typeb[ptr - type] = 0;
      }

      qtype = qtype_getcode(typeb);
      q = create_query(qdom, qtype, do_rd);

      /* set 'querying' message */
      lstData = stl_string("Quering ") + last_server + " for {" + qdom.tocstr() + "," + str_qtype(qtype) + "}";
      GtkWidget *li = gtk_list_item_new_with_label(lstData.c_str());
      gtk_container_add(GTK_CONTAINER(gt), li);
      gtk_widget_show(li);
      lstData += "\n";

      if (pthread_create(&tr, NULL, querythread, NULL) != 0) {
        /* failed! */
        delete q;
        throw PException("Failed to create query thread!");
      } else {
        query_busy = true;
        gtk_label_set_text(GTK_LABEL(GTK_BIN(lookup_widget(frmMain, "cmdQuery"))->child), "Stop!");
      }
    } catch (PException p) {
      GtkList *gt = (GtkList *)lookup_widget(frmMain, "lstAnswer");

      GtkWidget *li = gtk_list_item_new_with_label(
        (stl_string("*** ") + p.message).c_str());
      lstData = stl_string("*** ") + p.message + "\n";
      gtk_container_add(GTK_CONTAINER(gt), li);
      gtk_widget_show(li);
    }
  }
}

void
on_cmdOptions_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_menu_popup(GTK_MENU(mnuOptions), (GtkWidget*)NULL, (GtkWidget*)NULL,
                 (GtkMenuPositionFunc)NULL, NULL, 0, 0);
}


void
on_mnuOptionsAdvanced_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


gboolean
on_frmMain_destroy_event               (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  if (query_busy) query_stop();
  disable_answer_output = true;
  gtk_main_quit();
  return FALSE;
}


void
on_txtType_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{
  if (!txttype_flag) txttype_changed = true;
  if (strcmpi(gtk_entry_get_text(GTK_ENTRY(editable)), "(type in your own)") == 0)
    gtk_entry_set_text(GTK_ENTRY(editable), "");
}


void
on_txtDom_changed                      (GtkEditable     *editable,
                                        gpointer         user_data)
{
  if (!txttype_changed) {
    try {
      domainname dom = gtk_entry_get_text(GTK_ENTRY(lookup_widget(frmMain, "txtDom")));
      txttype_flag = true;
      if (dom >= "in-addr.arpa." || dom >= "ip6.int")
        gtk_entry_set_text(GTK_ENTRY(lookup_widget(frmMain, "txtType")), "PTR - Pointer");
      else      
        gtk_entry_set_text(GTK_ENTRY(lookup_widget(frmMain, "txtType")), orig_type.c_str());
      txttype_flag = false;
    } catch (PException p) { }
  }
}


void
on_mnuOptionsRecursion_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_mnuOptionsClipboard_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_selection_owner_set (lookup_widget(frmMain, "lstAnswer"),
                           gdk_atom_intern("CLIPBOARD", FALSE),
                           GDK_CURRENT_TIME);
  
}

void
on_lstAnswer_selection_get             (GtkWidget       *widget,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data)
{
  const char *timestr = lstData.c_str();

  gtk_selection_data_set (data, GDK_SELECTION_TYPE_STRING,
                          8, (const unsigned char*)timestr, strlen(timestr));

}

